(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.TEMP_UMD_MODULE_NAME_CHANGE_ME = factory());
}(this, (function () { 'use strict';

  class FirebaseArray {
      constructor(keys = [], values = [], listeners = []) {
          this.onChange = (eventHandler) => {
              const unsub = () => {
                  const eventIndex = this.listeners.indexOf(eventHandler);
                  this.listeners.splice(eventIndex, 1);
              };
              this.listeners.push(eventHandler);
              return unsub;
          };
          this.emit = () => {
              this.listeners.forEach(fn => fn(this));
          };
          this.get = () => {
              return {
                  keys: this.keys,
                  values: this.values
              };
          };
          this.emitAndGet = () => {
              this.emit();
              return this.get();
          };
          this.childAdded = (key, value, previousKey) => {
              if (previousKey === null) {
                  this.keys.unshift(key);
                  this.values.unshift(value);
                  return this.emitAndGet();
              }
              const previousKeyIndex = this.keys.indexOf(previousKey);
              if (previousKeyIndex === -1) {
                  this.keys.push(key);
                  this.values.push(value);
                  return this.emitAndGet();
              }
              this.keys.splice(previousKeyIndex + 1, 0, key);
              this.values.splice(previousKeyIndex + 1, 0, value);
              return this.emitAndGet();
          };
          this.childRemoved = (key) => {
              const keyIndex = this.keys.indexOf(key);
              this.keys.splice(keyIndex, 1);
              this.values.splice(keyIndex, 1);
              return this.emitAndGet();
          };
          this.childMoved = (key, value, previousKey) => {
              this.childRemoved(key);
              this.childAdded(key, value, previousKey);
              return this.emitAndGet();
          };
          this.childChanged = (key, value, previousKey) => {
              const keyIndex = this.keys.indexOf(key);
              if (keyIndex === -1) {
                  this.childAdded(key, value, previousKey);
              }
              else {
                  this.keys[keyIndex] = key;
                  this.values[keyIndex] = value;
              }
              return this.emitAndGet();
          };
          this.keys = keys;
          this.values = values;
          this.listeners = listeners;
      }
  }

  return FirebaseArray;

})));
